# coding:utf8

"""autogen thrift hubcode, handler, config

Usage:
    fasthrift <idls>
              [-p --pattern=<p>]
              [-o --outhub=<oh>]
              [-c --outconfig=<oc>]
              [-d --outhandler]

Options:
    -h --help     Show this screen.
    -p --pattern=<p>  Pattern(sync|async). [default: async]
    -o --outhub=<oh>   Output thrift hub module path. [default: ]
    -c --outconfig=<oc>    Output load thrift of fastweb path. [default: ]
    -d --handler=<hd>    Output handler template code. [default: handlers]
"""

import os
from glob import iglob

from fastweb.script import Script
from fastweb.spec.idl import Parser
from fastweb.util.log import recorder
from fastweb.accesspoint import docopt
from fastweb.util.python import write_file
from fastweb.exception import ThriftParserError, FastwebException


TAB = ' '*4


class ThriftCommand(Script):
    """一个thrift文件中只允许定义一个service，定义多个service会强制报错，不要在一个service中定义过多的function。
    遵循微服务的原则，将能够独立的部分拆分成独立的service"""

    def __init__(self):
        self._idl_parser = Parser()

    def gen_thrift_auxiliary(self):
        """生成与thrift相关的桩代码（hub code）和配置文件"""

        # 获取基础参数
        cwd = os.getcwd()
        args = docopt(__doc__)
        language = ''
        hub_suffix = ''
        idls = args['<idls>']
        hub_path = args['--outhub']
        config_path = args['--outconfig']
        handler_package = args['--handler']

        if args['--pattern'] == 'async':
            language = 'py:tornado'
            hub_suffix = 'async'
        elif args['--pattern'] == 'sync':
            language = 'py'
            hub_suffix = 'sync'
        else:
            recorder('ERROR', 'pattern not support')

        # 创建thrift桩代码
        hub_module_name, service_module_pathes, program = self._create_hub_package(idls, language, hub_path, hub_suffix)

        # 创建handler包目录
        try:
            handler_path = os.path.join(cwd, handler_package)
            os.mkdir(handler_path)
            recorder('INFO', 'create dir {path}'.format(path=handler_path))

            # 创建包__init__文件
            with open(os.path.join(handler_path, '__init__.py'), 'w') as f:
                f.write('"""generate from fasthrift"""')
        except OSError:
            pass

        # 创建handler模板
        for service in program:
            handler_module_name, handler_cls_name = self._create_handler(service, handler_path)
            handler_python_path = '{package}.{module}.{cls}'.format(package=handler_package,
                                                                    module=handler_module_name,
                                                                    cls=handler_cls_name)
        # 生成配置文件
        thrift_config_template = self._create_config_file(service_module_pathes, handler_python_path, program)

        # 创建配置文件
        if config_path:
            config_path = os.path.join(cwd, config_path)
            write_file(config_path, thrift_config_template)
            recorder('INFO', 'create config file {config}'.format(config=config_path))

        recorder('DEBUG',
                 'thrift hub module path: {hub}\n'
                 'thrift handler path: {handler}\n'
                 'thrift config path: {config}'.format(hub=os.path.join(cwd, hub_module_name),
                                                       config=config_path,
                                                       handler=handler_path))

    def _create_hub_package(self, idls, language, hub_path, hub_suffix):
        """创建thrift桩代码

        :return:
          - `service_module_pathes`: 每个thrift生成的桩代码python类路径
          - `service_handler_pathes`: 自动生成的handler的python类路径
        """

        cwd = os.getcwd()
        # package 名字中不能存在`-`，无法导入
        hub_module_name = 'fastweb_thrift_{hub_package}'.format(hub_package=hub_suffix)
        hub_path = os.path.join(hub_path, hub_module_name)
        hub_abspath = os.path.join(cwd, hub_path)

        # 创建桩代码的目录
        try:
            os.mkdir(hub_abspath)
            recorder('INFO', 'create dir {path}'.format(path=hub_abspath))
        except OSError:
            pass

        services = []
        service_module_pathes = []

        for idl in iglob(idls):
            # 检验+解析
            program = self._parse_idl(idl)
            if len(program) > 1:
                recorder('ERROR', 'each thrift file can only hold one service, you define {num} service in {idl}'.format(num=len(program), idl=idl))
                raise FastwebException

            service_package_name = idl.split('/')[-1].rstrip('.thrift')

            # 通过thrift命令生成桩代码
            command = 'thrift --gen {language} -out {out} {idl} '.format(language=language, idl=idl, out=hub_path)
            self.call_subprocess(command)
            for service in program:
                service_module_path = '{hub}.{package}.{module}'.format(hub=hub_module_name,
                                                                        package=service_package_name,
                                                                        module=service.name)
                service_module_pathes.append(service_module_path)
                services.append(service)

        return hub_module_name, service_module_pathes, services

    @staticmethod
    def _create_config_file(service_module_pathes, handler_python_path, program):
        """创建fastweb配置文件"""

        thrift_config_template = ''
        module_len = len(service_module_pathes)
        for i, service_module_pathes in enumerate(service_module_pathes):
            if i == 0:
                thrift_config_template += '; fastthrift gen template\n\n'

            thrift_config_template += '[service:{service}]\n' \
                                      'port =\n' \
                                      'thrift_module = {hub}\n' \
                                      'handlers = {handler}\n' \
                                      'active = yes'.format(hub=service_module_pathes,
                                                            handler=handler_python_path,
                                                            service=program[i].name)

            if i != module_len:
                thrift_config_template += '\n\n\n'

        recorder('INFO', 'fasthrift gen config')
        recorder('CRITICAL', '{config}'.format(config=thrift_config_template))
        return thrift_config_template

    @staticmethod
    def _create_handler(service, handler_path):
        """创建handler

        :parameter:
          - `service`: 服务对象
        """

        service_handler_cls = service.name + 'Handler'
        cls_template = '# coding:utf8\n\n"""generate by fasthrift"""\n\n\n' \
                       'from fastweb.service import ABLogic\n\n\n' \
                       'class {cls}(ABLogic):\n\n'.format(tab=TAB,
                                                          cls=service_handler_cls)
        func_template = '{tab}def {func}(self):\n{tab}{tab}pass\n\n'
        funcs_template = ''

        for func in service.functions:
            funcs_template += func_template.format(tab=TAB, func=func.name)

        handler_template = cls_template + funcs_template
        handler_path = os.path.join(handler_path, service.name+'.py')
        write_file(handler_path, handler_template)
        return service.name, service_handler_cls

    def _parse_idl(self, idl):
        """解析idl文件

        :parameter:
          - `idl`: idl文件"""
        try:
            program = self._idl_parser.parse_file(idl).definitions
            return program
        except ThriftParserError:
            recorder('ERROR', 'thrift file {idl} format error'.format(idl=idl))
            raise


def gen_thrift_auxiliary():
    ThriftCommand().gen_thrift_auxiliary()





from china_datasets.repo import DatasetRepo

from yaspin import yaspin
from yaspin.spinners import Spinners


def load_dataset(dataset, backend='s3'):
    """
    Loads the dataset.
    """
    repo = DatasetRepo()

    with yaspin(Spinners.moon, text="下载中...") as spinner:
        try:
            dataset = repo.get(dataset)
            spinner.text = "下载完成"
            spinner.ok("✅✅✅✅✅✅✅✅✅✅")
            return dataset
        except BaseException:
            spinner.text = "下载失败，请联系管理员<zhangchunyang_pri@126.com>"
            spinner.fail("🙀🙀🙀🙀🙀🙀🙀🙀🙀🙀")
            return



# coding:utf8

import logging
import traceback
import logging.config
from logging import getLogger

from termcolor import colored

import fastweb
from fastweb.exception import ParameterError
from fastweb.setting.default_logging import DEFAULT_LOGGING_SETTING


bSetupLogging = False
LOGGING_LEVEL = ['INFO',
                 'DEBUG',
                 'WARN',
                 'ERROR',
                 'CRITICAL',
                 'IMPORTANT']
COLORMAP = {
    'INFO': 'white',
    'DEBUG': 'green',
    'WARN': 'yellow',
    'ERROR': 'red',
    'CRITICAL': 'magenta',
    'IMPORTANT': 'cyan'
}


def setup_logging(setting):
    """加载logging配置

    :parameter:
      - `setting`:配置"""

    global bSetupLogging
    logging.basicConfig(level=logging.DEBUG)
    logging.config.dictConfig(setting)
    bSetupLogging = True


def recorder(level, msg):
    """日志记录

    load_recorder后会使用系统的日志handler,没有load_recorder会使用logging默认handler

    :parameter:
      - `level`:日志级别
      - `msg`:日志信息
    """

    if not bSetupLogging:
        from fastweb import app
        app.load_recorder()

    rec = fastweb.loader.app.system_recorder
    record(level, msg, rec)


def console_recorder(level, msg):
    """打印终端日志"""
    record(level, msg)


def set_record_color(colormap):
    """设置日志颜色"""
    global COLORMAP
    if list(colormap.keys()) in LOGGING_LEVEL:
        COLORMAP = colormap
    else:
        recorder('CRITICAL', 'colormap invalid, please fill it like {colormap}'.format(colormap=str(COLORMAP)))


def record(level, msg, r=None, extra=None):
    global COLORMAP
    level = level.upper()
    check_logging_level(level)

    if level == 'ERROR':
        msg = '{msg}\n\n{exeinfo}'.format(msg=msg, exeinfo=traceback.format_exc(), whole=4)

    logger_color = COLORMAP.get(level, 'white')

    if r:
        logger_func = getattr(r, 'info' if level.lower() == 'important' else level.lower())
        logger_func(colored(msg, logger_color, attrs=['bold']), extra=extra)
    else:
        print((colored(msg, logger_color, attrs=['bold'])))


def check_logging_level(level):
    """检查日志级别是否正确

    :parameter:
      - `level`:日志级别"""

    if level not in LOGGING_LEVEL:
        recorder('CRITICAL', 'please check logging level! right options {levels},'
                             ' current level is {level}'.format(level=level,
                                                                levels=str(LOGGING_LEVEL)))
        raise ParameterError



from china_datasets.backend import S3StorageBackend


from datasets import load_from_disk


class DatasetRepo(object):

    _backend_storage = None

    def __init__(self, backend="s3"):
        self._backend = backend
        self.set_backend(backend)

    def set_backend(self, backend="s3"):
        """设置存储后端"""

        if backend == "s3":
            self._backend = backend
            self._backend_storage = S3StorageBackend()

    def list(self):
        """列出数据集仓库中的数据集名称"""
        pass

    def get_dataset_info(self, dataset):
        """获取数据集信息"""
        pass

    def get(self, dataset):
        """获取数据集对象"""

        return load_from_disk(
            self._backend_storage.backend_uri.format(ds=dataset),
            fs=self._backend_storage,
        )



# coding=utf-8


import sys
from setuptools import setup, find_packages


kwargs = {}
install_requires = []
version = '1.0.0.0'

if sys.version_info < (3, 0):
    with open('README.md') as f:
        kwargs['long_description'] = f.read()

    with open('requirements.txt') as f:
        for require in f:
            install_requires.append(require[:-1])
    # install_requires.append('subprocess32')
elif sys.version_info > (3, 0):
    with open('README.md', encoding='utf-8') as f:
        kwargs['long_description'] = f.read()

    with open('requirements.txt', encoding='utf-8') as f:
        for require in f:
            install_requires.append(require[:-1])

# if sys.platform.startswith('linux'):
#     install_requires.append('readline')
# elif sys.platform.startswith('win'):
#     install_requires.append('pyreadline')

kwargs['install_requires'] = install_requires

setup(
    name='china-datasets',
    version=version,
    include_package_data=True,
    packages=find_packages(),
    package_data={'fast_datasets': []},
    entry_points={
        'console_scripts': [
            # 'fasthrift = fastweb.command.service.thrift:gen_thrift_auxiliary',
            # 'fast = fastweb.command.fast:main'
        ],
    },
    author='CYang',
    author_email='zhangchunyang_pri@126.com',
    description="china-datasets 是一个快速下载数据集，处理数据集的 python 软件包。",
    url="https://github.com/BSlience/china-datasets",
    long_description_content_type="text/markdown",
    **kwargs
)



# coding:utf8


"""call fastweb component

Usage:
    fast <configpath>

Options:
    -h --help     Show this screen.
    --args=<arg>  Remote task arguments.
"""


import os

from fastweb import app
from fastweb.accesspoint import docopt
from fastweb.util.tool import HistoryConsole
from fastweb.components import SyncComponents
from fastweb.util.log import console_recorder, colored


def main():
    cwd = os.getcwd()
    import sys
    sys.path.append(cwd)
    del sys
    args = docopt(__doc__)
    configpath = args['<configpath>']
    configpath = os.path.join(cwd, configpath)
    app.load_component(layout='service', backend='ini', path=configpath)
    components = SyncComponents()
    console_recorder('DEBUG', "[Fast command]\nyou can do anything like in fastweb")
    console = HistoryConsole(locals={'self': components})
    console.interact()



